#include "semantic.hh"


semantic *type_checker = new semantic();


/* Used to check that all functions contain return statements.
   Static means that it is only visible inside this file.
   It is set to false in do_typecheck() (ie, every time we start type checking
   a new block) and set to true if we find an ast_return node. See below. */
static bool has_return = false;


/* Interface for type checking a block of code represented as an AST node. */
void semantic::do_typecheck(symbol *env, ast_stmt_list *body)
{
    // Reset the variable, since we're checking a new block of code.
    has_return = false;
    if (body) {
        body->type_check();
    }

    // This is the only case we need this variable for - a function lacking
    // a return statement. All other cases are already handled in
    // ast_return::type_check(); see below.
    if (env->tag == SYM_FUNC && !has_return) {
        // Note: We could do this by overloading the do_typecheck() method -
        // one for ast_procedurehead and one for ast_functionhead, but this
        // will do... Hopefully people won't write empty functions often,
        // since in that case we won't have position information available.
        if (body != NULL) {
            type_error(body->pos) << "A function must return a value.\n";
        } else {
            type_error() << "A function must return a value.\n";
        }
    }
}

void semantic::check_bool(ast_expression *condition, string statement){
    if (condition->type_check() != integer_type) {
        type_error(condition->pos) << statement <<" predicate must be of integer "
                                   << "type.\n";
    }
    
}

void semantic::check_return(ast_expr_list* lst, sym_index return_type) {
        if (lst->last_expr->tag == AST_RETURN && lst->last_expr->type_check() != return_type) {
            type_error(lst->last_expr->pos) << "Type of function return does not match formal return type.\n";
            return;
        } else if (lst->preceding != NULL) {
            return check_return(lst->preceding, return_type);
        } else {
            return;
        }
}

void semantic::contains_return(ast_expr_list* lst) {
        if (lst->last_expr->tag == AST_RETURN) {
            type_error(lst->last_expr->pos) << "Procedure cant have return statements.\n";
            return;
        } else if (lst->preceding != NULL) {
            return contains_return(lst->preceding);
        } else {
            return;
        }
}
/* Compare formal vs. actual parameters recursively. */
void semantic::chk_param(ast_id *env,
                        parameter_symbol *formals,
                        ast_expr_list *actuals)
{
    /* Your code here */

    //printf("type_check done\n");

    if (formals->type != actuals->last_expr->type_check() ) {
        type_error(actuals->pos) << "Type discrepancy between formal and actual parameters.\n";             
        return;
    }
    //Recursion is done, no errors
    if (actuals->preceding == NULL && formals->preceding == NULL) {
        return;
    } else if (actuals->preceding == NULL){
        type_error(env->pos) << "More formal than actual parameters.\n";     
    } else if (formals->preceding == NULL) {
        type_error(env->pos) << "More actual than formal parameters.\n";     
    } else {
        return chk_param(env, formals->preceding, actuals->preceding);
    }
}


/* Check formal vs. actual parameters at procedure/function calls. */
void semantic::check_parameters(ast_id *call_id,
                                ast_expr_list *param_list)
{
    parameter_symbol* ps;
    //printf("check_pam\n");


    /* Your code here */
    symbol* s = sym_tab->get_symbol(call_id->sym_p);
    if (s->tag == SYM_FUNC) {
        ps = s->get_function_symbol()->last_parameter;
    } else {
        ps = s->get_procedure_symbol()->last_parameter;
    }

    if (ps == NULL && param_list != NULL) {
        //error
        type_error(call_id->pos) << "More actual than formal parameters.\n";     
    } else if (param_list == NULL && ps != NULL) {
        //error
        type_error(call_id->pos) << "More formal than actual parameters.\n";     
    } else if (ps == NULL && param_list == NULL) {
        return;
    } else {
        type_checker->chk_param(call_id, ps, param_list);
    }



    //parameter_symbol* ps = s->get_parameter_symbol();


    
    //chk_params()
    //if false
    //type error
}



/* We overload this method for the various ast_node subclasses that can
   appear in the AST. By use of virtual (dynamic) methods, we ensure that
   the correct method is invoked even if the pointers in the AST refer to
   one of the abstract classes such as ast_expression or ast_statement. */
sym_index ast_node::type_check()
{
    fatal("Trying to type check abstract class ast_node.");
    return void_type;
}

sym_index ast_statement::type_check()
{
    fatal("Trying to type check abstract class ast_statement.");
    return void_type;
}

sym_index ast_expression::type_check()
{
    fatal("Trying to type check abstract class ast_expression.");
    return void_type;
}

sym_index ast_lvalue::type_check()
{
    fatal("Trying to type check abstract class ast_lvalue.");
    return void_type;
}

sym_index ast_binaryoperation::type_check()
{
    fatal("Trying to type check abstract class ast_binaryoperation.");
    return void_type;
}

sym_index ast_binaryrelation::type_check()
{
    fatal("Trying to type check abstract class ast_binaryrelation.");
    return void_type;
}



/* Type check a list of statements. */
sym_index ast_stmt_list::type_check()
{
    //printf("stmt list type check\n");
    if (preceding != NULL) {
        preceding->type_check();
    }
    if (last_stmt != NULL) {
        last_stmt->type_check();
    }
    return void_type;
}


/* Type check a list of expressions. */
sym_index ast_expr_list::type_check()
{
     //printf("expr_list\n");
    /* Your code here */
    if (preceding != NULL) {
        preceding->type_check();
    }
    if (last_expr != NULL) {
        last_expr->type_check();
    }
    return void_type;
}



/* Type check an elsif list. */
sym_index ast_elsif_list::type_check()
{
    /* Your code here */
     //printf("elf_list\n");
    if (preceding != NULL) {
        preceding->type_check();
    }
    if (last_elsif != NULL) {
        last_elsif->type_check();
    }
    return void_type;
}


/* "type check" an indentifier. We need to separate nametypes from other types
   here, since all nametypes are of type void, but should return an index to
   itself in the symbol table as far as typechecking is concerned. */
sym_index ast_id::type_check()
{
    //printf("super id typecheck\n");
    if (sym_tab->get_symbol(sym_p)->tag != SYM_NAMETYPE) {
        return type;
    }
    return sym_p;
}


sym_index ast_indexed::type_check()
{
    /* Your code here */
    //printf("indexed,baby\n");
    sym_index s_type = index->type_check();
    if (s_type != integer_type){
         type_error(pos) <<"Index must be of type integer "
                                   << "type.\n";
    }
    symbol *s = sym_tab->get_symbol(id->sym_p);
    return s->type;
}



/* This convenience function is used to type check all binary operations
   in which implicit casting of integer to real is done: plus, minus,
   multiplication. We synthesize type information as well. */
sym_index semantic::check_binop1(ast_binaryoperation *node)
{
    /* Your code here */
    //printf("binop1\n");
    
    if (node->left->type_check() == real_type && node->right->type_check() == integer_type) {
        node->right = (new ast_cast(node->right->pos, node->right))->expr;
        return real_type;
    } else if (node->right->type_check() == real_type && node->left->type_check() == integer_type) {
        node->left = (new ast_cast(node->left->pos, node->left))->expr;
        return real_type;
    } else if (node->left->type_check() == real_type && node->right->type_check() == real_type) {
        return real_type;
    }
    return integer_type; // You don't have to use this method but it might be convenient
}

sym_index ast_add::type_check()
{
    /* Your code here */
    return type_checker->check_binop1(this);
    
}

sym_index ast_sub::type_check()
{
    /* Your code here */
    return type_checker->check_binop1(this);

}

sym_index ast_mult::type_check()
{
    /* Your code here */
    return type_checker->check_binop1(this);
}

/* Divide is a special case, since it always returns real. We make sure the
   operands are cast to real too as needed. */
sym_index ast_divide::type_check()
{
    /* Your code here */
    if (this->left->type_check() == integer_type) {
        this->left = (new ast_cast(this->left->pos, this->left))->expr;
    } 
    if (this->right->type_check() == integer_type) {
        this->left = (new ast_cast(this->left->pos, this->left))->expr;
    }
    return real_type;
}



/* This convenience method is used to type check all binary operations
   which only accept integer operands: AND, OR, MOD, DIV.
   The second argument is the name of the operator, so we can generate a
   good error message.
   All of these return integers, so we synthesize that.
   */
sym_index semantic::check_binop2(ast_binaryoperation *node, string s)
{
    /* Your code here */
    //printf("bin2\n");

    if (node->left->type_check() == real_type) {
        type_error(node->left->pos) << s <<" left argument must be of integer "
                                   << "type.\n";
    } else if (node->right->type_check() == real_type) {
        type_error(node->right->pos) << s <<" right argument must be of integer "
                                   << "type.\n";
    }
    return integer_type;
}

sym_index ast_or::type_check()
{
    /* Your code here */
    return type_checker->check_binop2(this, "OR");
}

sym_index ast_and::type_check()
{
    /* Your code here */
    return type_checker->check_binop2(this, "AND");

}

sym_index ast_idiv::type_check()
{
    /* Your code here */
    return type_checker->check_binop2(this, "DIV");
}

sym_index ast_mod::type_check()
{
    /* Your code here */
    return type_checker->check_binop2(this, "MOD");
}



/* Convienience method for all binary relations, since they're all typechecked
   the same way. They all return integer types, 1 = true, 0 = false. */
sym_index semantic::check_binrel(ast_binaryrelation *node)
{
    /* Your code here */
    if (node->left->type_check() == real_type && node->right->type_check() == integer_type) {
        node->right = (new ast_cast(node->right->pos, node->right))->expr;
    } else if (node->right->type_check() == real_type && node->left->type_check() == integer_type) {
        node->left = (new ast_cast(node->left->pos, node->left))->expr;
    }
    return integer_type;
}

sym_index ast_equal::type_check()
{
    /* Your code here */
    return type_checker->check_binrel(this);
}

sym_index ast_notequal::type_check()
{
    /* Your code here */
    return type_checker->check_binrel(this);
}

sym_index ast_lessthan::type_check()
{
    /* Your code here */
    return type_checker->check_binrel(this);
}

sym_index ast_greaterthan::type_check()
{
    /* Your code here */
    return type_checker->check_binrel(this);
}



/*** The various classes derived from ast_statement. ***/

sym_index ast_procedurecall::type_check()
{
    /* Your code here */

    type_checker->check_parameters(id, parameter_list);

    if (parameter_list != NULL) {
        type_checker->contains_return(parameter_list);
    }
    return void_type;
}


sym_index ast_assign::type_check()
{
    /* Your code here */
    //printf("ast assign type check\n");
    if (lhs->type_check() == integer_type & rhs->type_check() == real_type) {
        type_error(rhs->pos) << "Can't assign a real value to an integer variable.\n";
    }
    return void_type;
}


sym_index ast_while::type_check()
{
    type_checker->check_bool(condition, "while");

    if (body != NULL) {
        body->type_check();
    }
    return void_type;
}


sym_index ast_if::type_check()
{
    //printf("ast_if\n");
    /* Your code here */
    type_checker->check_bool(condition, "if");

    if (body != NULL) {
        //printf("ast_if body type  check\n");
        body->type_check();
    }
    return void_type;
}


sym_index ast_return::type_check()
{
    // This static global (meaning it is global for all methods in this file,
    // but not visible outside this file) variable is reset to 0 every time
    // we start type checking a new block of code. If we find a return
    // statement, we set it to 1. It is checked in the do_typecheck() method
    // of semantic.cc.
    has_return = true;

    // Get the current environment. We don't yet know if it's a procedure or
    // a function.
    symbol *tmp = sym_tab->get_symbol(sym_tab->current_environment());
    if (value == NULL) {
        // If the return value is NULL,
        if (tmp->tag != SYM_PROC)
            // ...and we're not inside a procedure, something is wrong.
        {
            type_error(pos) << "Must return a value from a function.\n";
        }
        return void_type;
    }

    sym_index value_type = value->type_check();

    // The return value is not NULL,
    if (tmp->tag != SYM_FUNC) {
        // ...so if we're not inside a function, something is wrong too.
        type_error(pos) << "Procedures may not return a value.\n";
        return void_type;
    }

    // Now we know it's a function and can safely downcast.
    function_symbol *func = tmp->get_function_symbol();

    // Must make sure that the return type matches the function's
    // declared return type.
    if (func->type != value_type) {
        type_error(value->pos) << "Bad return type from function.\n";
    }

    return void_type;
}


sym_index ast_functioncall::type_check()
{
    /* Your code here */
    //printf("function call\n");
    type_checker->check_parameters(id, parameter_list);
    
    sym_index t = sym_tab->get_symbol(id->sym_p)->type;
    if (parameter_list->last_expr != NULL && has_return) {
        type_checker->check_return(parameter_list, t);
    }
    return t;
}

sym_index ast_uminus::type_check()
{
    /* Your code here */
    //printf("uminustypecheck\n");
    if (this->expr->type_check() == real_type) {
        type_error(this->pos) << "Cannot make negative of real_type.\n";
    }
    return integer_type;
}

sym_index ast_not::type_check()
{
    /* Your code here */
    type_checker->check_bool(expr, "not");

    return void_type;
}


sym_index ast_elsif::type_check()
{
    /* Your code here */
    type_checker->check_bool(condition, "if");

    if (body != NULL) {
        body->type_check();
    }

    return void_type;
}



sym_index ast_integer::type_check()
{
    return integer_type;
}

sym_index ast_real::type_check()
{
    return real_type;
}
